@@ -18,18 +18,8 @@
 static int lvremove_single(struct cmd_context *cmd, struct logical_volume *lv,
 			   void *handle __attribute__((unused)))
 {
-	struct logical_volume *origin;
-
-	/*
-	 * If this is a sparse device, remove its origin too.
-	 */
-        if (lv_is_cow(lv) && lv_is_virtual_origin(origin = origin_from_cow(lv)))
-                lv = origin;
-
-	if (!lv_remove_with_dependencies(cmd, lv, (force_t) arg_count(cmd, force_ARG), 0)) {
-		stack;
-		return ECMD_FAILED;
-	}
+	if (!lv_remove_with_dependencies(cmd, lv, (force_t) arg_count(cmd, force_ARG), 0))
+		return_ECMD_FAILED;
 
 	return ECMD_PROCESSED;
 }
